-- Small things that didn't fit into their own module

local tracked = {
	{dep = true, var = "kmp_bumpspark", pref = 1},
	{dep = true, var = "kmp_hardsneakers", pref = 1},
	{dep = true, var = "kmp_extendflashtics", pref = 0},
	{dep = true, var = "kmp_slowspinout", pref = 1},
	{dep = true, var = "kmp_orbitlimit", pref = 0},
	{dep = true, var = "kmp_hyudoro", pref = 0},
	{dep = true, var = "kmp_orbinautfuse", pref = 0},
	{dep = true, var = "kmp_shrinkduration", pref = 0},
	{dep = true, var = "kmp_haste", pref = 1},
	{dep = true, var = "respawndelay", pref = 1},
	{dep = true, var = "shrink", pref = 0},
	{var = "am_enable", pref = 0},
}
rawset(_G, "JUICEBOX_restorecvars", CV_RegisterVar({
	name = "jb_restorecvars",
	defaultvalue = "On",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff
}))
local function jb_setup()
	if not server then return end
	if (consoleplayer) and (consoleplayer != server) then return end
	-- CONS_Printf(server, ">> Beginning Juicebox cvar setup.")
	-- CONS_Printf(server, ">> You may see this multiple times, the console is slow and config is weird.")
	for k, v in ipairs(tracked)
		if CV_FindVar(v.var)
			if CV_FindVar(v.var).value != v.pref
				CONS_Printf(server, "* Changing "..v.var.." to "..v.pref.."...")
				tracked[k].saved = CV_FindVar(v.var).value
				COM_BufAddText(server, v.var.." "..v.pref..";wait 2")
			end
		else
			if v.dep then
				CONS_Printf(server, "* Couldn't find "..v.var.."! You're probably missing KMP, balance will be way worse.")	
			end
		end
	end
end
local function jb_restore()
	if not server then return end
	if (consoleplayer) and (consoleplayer != server) then return end
	if not JUICEBOX_restorecvars.value then
		CONS_Printf(server, ">> "..JUICEBOX_restorecvars.name.." is disabled -- leaving config alone!")
		return
	end
	CONS_Printf(server, ">> Reverting changes Juicebox made to cvars during setup.")
	CONS_Printf(server, ">> To disable this and handle changes yourself, set jb_restorecvars Off.")
	CONS_Printf(server, ">> If you'd like to use this, but it's changing the wrong values, use jb_reset after your configs are finished.")
	for k, v in ipairs(tracked)
		if v.saved != nil
			if v.var == "respawndelay" and JUICEBOX_forcefastrespawn.value then
				CONS_Printf(server, "* Force Fast Respawn is on, setting respawndelay 0!")
				COM_BufAddText(server, "respawndelay 0;wait 2")
				continue
			end
			CONS_Printf(server, "* Changing "..v.var.." back to "..v.saved.."...")
			COM_BufAddText(server, v.var.." "..v.saved..";wait 2")
			v.saved = nil
		end
	end
end
rawset(_G, "JUICEBOX", CV_RegisterVar({
	name = "juicebox",
	defaultvalue = "On",
	flags = CV_NETVAR|CV_CALL,
	possiblevalue = CV_OnOff,
	func = function(var)
		if var.value then
			jb_setup()
		else
			jb_restore()
		end
	end
}))
local function jb_reset(p)
	CONS_Printf(p, ">> Clearing saved cvar preferences.")
	for key, val in ipairs(tracked)
		if val.saved != nil then
			CONS_Printf(p, "* Clearing: "..val.var.." "..val.saved)
			val.saved = nil
		end
	end
	if JUICEBOX.value then
		CONS_Printf(p, "* Juicebox is ON, forcing setup to run.")
		jb_setup()
	else
		CONS_Printf(p, "* Juicebox is OFF, not running setup.")
	end
end
COM_AddCommand("jb_reset", jb_reset, 1)
rawset(_G, "JUICEBOX_forcefastrespawn", CV_RegisterVar({
	name = "fastrespawn",
	defaultvalue = "Off",
	flags = CV_NETVAR,
	possiblevalue = CV_OnOff
}))
local JUICEBOX_tips = CV_RegisterVar({
	name = "jb_tips",
	defaultvalue = "On",
	possiblevalue = CV_OnOff
})
print("| Juicebox \"Guava\" - a refreshing take on racing")
print("| by Tyron - aj@worldsbe.st")
print("| ")
print("| ~SPECIAL THANKS~")
print("| Additional sounds - Runa, Lazuli, Eebrozgi")
print("| Additional graphics - Chengi, Eebrozgi")
print("| Some original concepts - Chromatian, VelocitOni")
print("| Teching prototypes - Wolfs, Barkley")
print("| Code contributions - Lat, Snu, Wolfs")
print("| Testing and support - ALL Neps illuminati!, ALL KCS wizards")

local init = false
addHook("NetVars", function(net)
	init = net(init)
end)
addHook("ThinkFrame", function()
	if leveltime == 0 then return end
	if not init then
		if JUICEBOX.value then
			jb_setup()
		end
		if HOSTMOD then
			HM_Scoreboard_AddMod({disp = "\141Juicebox", var = "juicebox"})
			HM_Scoreboard_AddMod({disp = "\141hyuu.cc/j", var = "juicebox"})
		end
		init = true
	end
end)

local jb_stackcut = {value = 60}
local jb_stackmax = {value = 90}
local jb_stackscale = {value = 7}

rawset(_G, "JUICEBOX_Stack", function(p, boost, isgateboost)
	local taxifromtr = (9 - p.kartspeed) + (p.kartweight - 1)
	local stackcut = max(jb_stackmax.value - jb_stackscale.value*taxifromtr, jb_stackcut.value)
	--print(taxifromtr, stackcut)
	--print("STACK: "..boost.." + "..p.kartstuff[k_driftboost])
	if p.kartstuff[k_driftboost] > boost then -- existing boost is getting cut, so we need to cut gateboost visual if any
		p.gateboostflash = FixedInt(FixedMul($*FRACUNIT, stackcut*FRACUNIT/100))
	end
	if isgateboost and p.kartstuff[k_driftboost] < boost then -- won't be cut, add wholesale
		p.gateboostflash = $ + boost
	elseif isgateboost then -- will be cut
		p.gateboostflash = $ + FixedInt(FixedMul(boost*FRACUNIT, stackcut*FRACUNIT/100))
	end
	p.kartstuff[k_driftboost] = max($, boost) + FixedInt(FixedMul(min($, boost)*FRACUNIT, stackcut*FRACUNIT/100))
	--print("= "..p.kartstuff[k_driftboost])
end)

rawset(_G, "JUICEBOX_TechPenalty", function(p, consolation)
	p.gatedecay = $ or 100
	p.techpenalty = $ or -9999
	if abs(p.techpenalty - leveltime) >= 5*TICRATE
		if consolation
			p.gatedecay = 6*$/9
		else
			p.gatedecay = 6*$/11
		end
		p.gatedecay = max(10, $)
	end
	p.techpenalty = leveltime
end)

rawset(_G, "JUICEBOX_dp", 0)
rawset(_G, "JUICEBOX_renderer", 0)
hud.add(function(v, p, c)
	JUICEBOX_dp = p
	JUICEBOX_renderer = v.renderer()
end)

rawset(_G, "JUICEBOX_dwatch", {})
rawset(_G, "JUICEBOX_players", 0)
rawset(_G, "JUICEBOX_splitscreen", 1)
rawset(_G, "JUICEBOX_weightadjust", 0)
rawset(_G, "JUICEBOX_local", function()
	for dp, n in displayplayers.iterate
		if (p != dp) then continue end
		return n
	end
	return false
end)
rawset(_G, "JUICEBOX_StartLocalSound", function(sfx, p)
	if p.JBlocal
		S_StartSound(nil, sfx, p)
	end
end)
rawset(_G, "JUICEBOX_StartLocalSoundWithScaledVolumeInThisGorillaSpaghettiSplitscreenSystem", function(sfx, p) -- I don't want to talk about it, okay
	if p.JBlocal
		if JUICEBOX_splitscreen then
			S_StartSoundAtVolume(nil, sfx, 255/JUICEBOX_splitscreen, p)
		else -- HOW?
			S_StartSoundAtVolume(nil, sfx, 255, p)
		end
	end
end)

local tips = {
	[[We host sessions every sunday. Do !discord to join us!]],
	[[Need help with emotes? Do emotehelp in console to see all the commands!]],
	[[Got a suggestion? Join the discord server and let us know]],
	[[Annoyed of emotes? Do emotesoff in console]],
	[[Trans rights are human rights!]],
	[[We sometimes stream over @ youtube.com/ChaoticNature]],
	[[We hope you enjoy your stay in Vibe Check Staion]],
	[[Dont know to trick? Press your break button when jumping!]],
	[[If someone is breaking a rule, contact @ Chaotic Nature#8852]],
	
}

addHook("ThinkFrame", function()
	if not JUICEBOX.value then 
		if leveltime == 1 then
			if JUICEBOX_forcefastrespawn.value then -- hack, xd
				COM_BufAddText(server, "respawndelay 0")
			else
				COM_BufAddText(server, "respawndelay 1")
			end
		end
		return 
	end
	if leveltime == 0 then return end
	if leveltime == 1 then
		jb_setup()
		for p in players.iterate
			p.juicetips = 0
		end
	end
	JUICEBOX_dwatch = {}
	JUICEBOX_players = 0
	JUICEBOX_splitscreen = 0
	for p in players.iterate do
		if not (p.mo and p.mo.valid and (not p.spectator)) continue end
		if p.exiting or (p.lives == 0) then
			p.juicetips = $ or 0
			p.juicetips = $ + 1
			p.juicetipreceived = $ or {}
			if p.juicetips == 2*TICRATE then
				local pendingtip = (leveltime%#tips)
				local received = 0
				for key, val in pairs(p.juicetipreceived)
					received = $ + 1
				end
				if received < #tips then
					repeat
						pendingtip = ($+1)%#tips
					until not p.juicetipreceived[pendingtip]
					p.juicetipreceived[pendingtip] = true
					if not j_turntips.value then return end
					if JUICEBOX_tips.value then
						chatprintf(p, "\140*Sundae Rakieta: "..tips[pendingtip+1], true)
					end
				end
			end
		end
		
		JUICEBOX_players = $ + 1
		p.JBlocal = false
		for dp, n in displayplayers.iterate
			if (p != dp) then continue end
			p.JBlocal = true
			JUICEBOX_splitscreen = $ + 1
			break
		end
		p.gateboostflash = $ or 0
		p.gateboostflash = max(0, $ - 1)
		if p.playerstate != PST_LIVE then p.gateboostflash = 0 end
		if p.gateboostflash and not (leveltime%3) and (not p.kartstuff[k_spinouttimer]) then
			local fug = P_SpawnGhostMobj(p.mo)
			fug.colorized = true
			if p.JBlocal then
				fug.flags2 = $ & ~MF2_DONTDRAW
			else
				fug.flags2 = $|MF2_DONTDRAW
			end
		end
	end
	JUICEBOX_weightadjust = 9 - (JUICEBOX_players - 6)/2
end)

rawset(_G, "JUICEBOX_netdebug", CV_RegisterVar({
	name = "jb_netdebug",
	defaultvalue = "Off",
	flags = CV_NETVAR|CV_NOSHOWHELP,
	possiblevalue = CV_OnOff
}))

rawset(_G, "JUICEBOX_reducemotion", CV_RegisterVar({
	name = "jb_reducemotion",
	defaultvalue = "Off",
	possiblevalue = CV_OnOff
}))